#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 18:05:54
LastEditors:  
LastEditTime: 2019-07-31 18:05:54
Description:  
"""

# GAS = 2000000               # 默认的gas值
# GAS_PRICE = 22000000000     # 默认的gas_price值
GAS = 100000               # 默认的gas值
GAS_PRICE = 100000     # 默认的gas_price值
PROVIDER = 'http://localhost:7545'



# -*- coding: utf-8 -*-

import time
import sys
import re
import json
import logging
from web3 import Web3, HTTPProvider, IPCProvider, WebsocketProvider
from web3.contract import ConciseContract
import eth_account, eth_utils
from pycontractsdk import keys, utils, config


log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
console = logging.StreamHandler()
console.setFormatter(formatter)
log.addHandler(console)


class Contract(object):
    """
    基础协约
    定义一些基础的方法
    """
    _w3 = Web3()
    _provider = ''                  # 哪种方式连接的以太坊(http://localhost:7545 | ws://localhost:7545 | /foo/ba/x.ipc)
    _contract_address = ''          # 智能协约的address
    _abi = ''                       # 智能协约的abi
    _private_key = ''               # 用于签名的 private key

    _gas = config.GAS               # 默认的gas值
    _gas_price = config.GAS_PRICE   # 默认的gas_prise值

    def __init__(self, provider=config.PROVIDER, timeout=60, *args, **kwargs):
        self._provider = provider
        if re.match('^http://.*', provider):
            self._w3 = Web3(HTTPProvider(provider, request_kwargs={'timeout': timeout}))
        elif re.match('^ws://.*', provider):
            self._w3 = Web3(WebsocketProvider(provider))
        else:
            self._w3 = Web3(IPCProvider(provider))

        self._contract_address = kwargs.pop('contract_address', '')
        self._abi = kwargs.pop('abi', '')
        self._private_key = kwargs.pop('private_key', '')

        self._gas = kwargs.pop('gas', self._gas)
        self._gas_price = kwargs.pop('gas_prise', self._gas_price)

    @property
    def w3(self):
        return self._w3

    @property
    def contract_address(self):
        return self._contract_address

    @contract_address.setter
    def contract_address(self, contract_address):
        self._contract_address = contract_address

    @property
    def abi(self):
        return self._abi

    @abi.setter
    def abi(self, abi):
        self._abi = abi

    @property
    def private_key(self):
        return self._private_key

    @private_key.setter
    def private_key(self, private_key):
        self._private_key = private_key

    def get_receipt(self, txid):
        return self._w3.eth.getTransactionReceipt(txid)

    def check_receipts(self, txid):
        """
        根据交易id获得receipts
        :param txid: 交易ID
        :return:
        """
        try:
            # receipt = self._w3.eth.getTransactionReceipt(txid)
            receipt = self._w3.eth.waitForTransactionReceipt(txid)
        except Exception as e:
            print(e)
            receipt = None
        return receipt

    def wait_for_tx(self, txid, second=120):
        """等待交易完成"""
        sec = 0
        timeout = False
        receipt = self.check_receipts(txid)
        while (receipt == None) or (receipt != None and receipt.blockHash == None):
            time.sleep(1)
            sec += 1
            print('.', end='')
            sys.stdout.flush()
            receipt = self.check_receipts(txid)
            if sec > second:
                timeout = True
                log.info('{}  {}\'s Transaction timeout'.format(txid,sec))
                log.info('Transaction timeout!')
                break
        print('seconds:[{}]'.format(sec))
        if receipt != None and receipt.blockHash != None:
            print('receipt: ' + str(dict(receipt)))
            return True, receipt
        return False, receipt if timeout else True, receipt

    def is_tx_sucess(self, txid):
        """
        查询 txid 上链是否成功
        :param txid:
        :return:
        """
        receipt = self.check_receipts(txid)
        if (receipt == None) or (receipt != None and receipt.blockHash == None):
            return False, receipt
        if receipt != None and receipt.blockHash != None:
            return True, receipt
        return False, receipt

    def get_address_nonce(self, address):
        """
        获得 address 对应的 nonce 值
        :param address:
        :return:
        """
        web3 = self._w3
        if eth_utils.is_address(address):
            nonce = web3.eth.getTransactionCount(eth_utils.to_checksum_address(address),
                                                 block_identifier=web3.eth.defaultBlock)
        else:
            raise ValueError("%s is not a address" % address)
        return nonce

    def get_private_key_nonce(self, private_key):
        """
        获得 private_key 对应的 nonce 值
        :param private_key:
        :return:
        """
        address = keys.privatekey_to_address(private_key)
        return self.get_address_nonce(address)

    def get_last_block(self):
        return self._w3.eth.getBlock()

    def last_block_number(self):
        """ 获得当前最后一个块的块号 """
        return self._w3.eth.blockNumber

    def get_transaction(self, tx_hash):
        return self._w3.eth.getTransaction(tx_hash)

    def get_transaction_receipt(self, tx_hash):
        return self._w3.eth.getTransactionReceipt(tx_hash)

    def get_contract(self):
        """ 用于调用 function 进行写（会进行上链操作） """
        web3 = self._w3
        starcoin_address_checksum = keys.to_check_sum_address(self._contract_address)
        contract = web3.eth.contract(abi=self._abi, address=starcoin_address_checksum)
        return contract

    def get_concise_contract(self):
        """ 用于读取本地状态数据的协约 """
        web3 = self._w3
        starcoin_address_checksum = keys.to_check_sum_address(self._contract_address)
        contract = web3.eth.contract(abi=self._abi, address=starcoin_address_checksum, ContractFactoryClass=ConciseContract)
        return contract

    def send_eth(self, sender_private_key, receiver_address, eth):
        """
        转eth
        :param sender_private_key: 发送者的私钥
        :param receiver_address: 接收者的address
        :param eth: 具体的ETH数量
        :param gas:
        :return:
        """
        web3 = self._w3
        nonce = self.get_private_key_nonce(sender_private_key)
        signed_txn = web3.eth.account.signTransaction(dict(
            nonce=nonce,
            gas=self._gas,
            gasPrice=self._gas_price,
            to=web3.toChecksumAddress(receiver_address),
            value=eth,
            # value=20000 * 10 ** 18,
            data='give {} {} eth'.format(receiver_address, eth).encode('utf8'),
            # data=b'',
        ),
            sender_private_key,
        )
        tx_hash = web3.eth.sendRawTransaction(signed_txn.rawTransaction)
        # 等待上链完成
        tx_hash_hex = eth_utils.encode_hex(tx_hash)
        (tf, receipt) = self.wait_for_tx(tx_hash_hex)
        log.debug(receipt)
        return tf, tx_hash_hex, receipt

    def send_row_transaction(self, row_transaction):
        """
        将签名的数据进行上链操作
        :param row_transaction:
        :return:
        """
        web3 = self._w3
        while True:
            try:
                tx_hash = web3.eth.sendRawTransaction(row_transaction)
            except ValueError as e:
                json_str = str(e)
                log.info(json_str)
                error_msg = json.loads(json.dumps(eval(json_str)))
                code = error_msg['code']
                if code == -32010:
                    time.sleep(1)
                    continue
            break
        tx_hash_hex = eth_utils.encode_hex(tx_hash)
        # (tf, receipt) = self.wait_for_tx(tx_hash_hex)
        # log.debug(receipt)
        receipt = self.get_receipt(tx_hash_hex)
        upload_success = utils.is_success_upload(receipt)
        return upload_success, tx_hash_hex, receipt

    def send_row_transaction_wait(self, row_transaction):
        """
        将签名的数据进行上链操作
        :param row_transaction:
        :return:
        """
        (upload_success, tx_hash_hex, receipt) = self.send_row_transaction(row_transaction)
        (upload_success, receipt) = self.wait_for_tx(tx_hash_hex)
        log.debug(receipt)
        return upload_success, tx_hash_hex, receipt

    def upload_function(self, contract_function, private_key):
        """
        将协约的方法进行签名和上链操作
        :param contract_function: 智能协约中对应的方法
        :param private_key: 签名用的private key
        :return:
        """
        web3 = self._w3
        nonce = self.get_private_key_nonce(private_key)
        # print('nonce:{}'.format(nonce))
        log.debug('nonce:{}'.format(nonce))

        tx_info = contract_function.buildTransaction(
            {'nonce': nonce,
             'gasPrice': int(self._gas_price) if isinstance(self._gas_price, str) else self._gas_price,
             'gas': int(self._gas) if isinstance(self._gas, str) else self._gas}
        )
        signed_txn = web3.eth.account.signTransaction(tx_info, private_key)
        tx_hash = web3.eth.sendRawTransaction(signed_txn.rawTransaction)
        tx_hash_hex = eth_utils.encode_hex(tx_hash)
        receipt = self.get_receipt(tx_hash_hex)
        # (tf, receipt) = self.wait_for_tx(tx_hash_hex)
        upload_success = utils.is_success_upload(receipt)
        return upload_success, tx_hash_hex, receipt

    def upload_function_wait(self, contract_function, private_key):
        """
        将协约的方法进行签名和上链操作 获得txid (会一直循环操作)
        如果pool里面已经有相同的txid，就会一直循环到下一个nonce值，在提交
        :param contract_function: 智能协约中对应的方法
        :param private_key: 签名用的private key
        :return:
        """
        web3 = self._w3
        while True:
            try:
                nonce = self.get_private_key_nonce(private_key)
                log.info("nonce:{}".format(nonce))
                tx_info = contract_function.buildTransaction(
                    {'nonce': nonce,
                     'gasPrice': int(self._gas_price) if isinstance(self._gas_price, str) else self._gas_price,
                     'gas': int(self._gas) if isinstance(self._gas, str) else self._gas }
                )
                signed_txn = web3.eth.account.signTransaction(tx_info, private_key)
                tx_hash = web3.eth.sendRawTransaction(signed_txn.rawTransaction)
            except ValueError as e:
                json_str = str(e)
                log.info(json_str)
                error_msg = json.loads(json.dumps(eval(json_str)))
                code = error_msg['code']
                if code == -32010:
                    time.sleep(1)
                    continue
            break
        tx_hash_hex = eth_utils.encode_hex(tx_hash)
        (tf, receipt) = self.wait_for_tx(tx_hash_hex)
        log.debug(receipt)
        return tf, tx_hash_hex, receipt

    def call_function_wait(self, function_name, *args, **kwargs):
        """
        等待上链成功
        :param function_name:   调用的协约方法
        :param args:            协约方法对应的参数，有顺序。
        :param kwargs:
        :return:
        """
        return self.call_function(function_name, True, *args)

    def call_function_nowait(self, function_name, *args, **kwargs):
        """
        不等待 直接 返回
        :param function_name:   调用的协约方法
        :param args:            协约方法对应的参数，有顺序。
        :param kwargs:
        :return:
        """
        return self.call_function(function_name, False, *args)

    def call_function(self, function_name, wait=True, *args, **kwargs):
        """
        调用智能协约的通用方法（会进行上链操作）
        :param function_name:   调用的协约方法
        :param wait:            是否等待receipt的返回
        :param args:            协约方法对应的参数，有顺序。
        :param kwargs:
        :return:
        """
        contract = self.get_contract()
        # 使用反射的方法调用web3
        function = getattr(contract.functions, function_name)(*args)
        if wait:
            tf, tx_hash, receipt = self.upload_function_wait(function, self._private_key)
        else:
            tf, tx_hash, receipt = self.upload_function(function, self._private_key)
        return tf, tx_hash, receipt

    def call_concise_function(self, function_name, *args, **kwargs):
        """
        调用智能协约的通用方法（不会上链，只是读取本地状态数据的协约）
        :param function_name: 调用的协约方法
        :param args: 协约方法对应的参数，有顺序。
        :param kwargs:
        :return:
        """
        contract = self.get_concise_contract()
        # 使用反射的方法调用web3
        value = getattr(contract, function_name)(*args)
        return value

    def getBalance(self, address, block_identifier=None):
        """
        获得地址的eth数量
        :param address:             需要查询的地址
        :param block_identifier:    第几个块上的数据，默认是最后一个块
        :return:
        """
        return self.w3.eth.getBalance(address, block_identifier)

    def __str__(self):
        return "connect provider : %s" % self._provider











#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-10 08:51:15
LastEditors:  
LastEditTime: 2019-04-10 08:51:15
Description:  
"""

class kk:
    def __init__(self):
        pass
    def ts(self):
        print("tsts")


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 工具类
#  

"""

import json


def get_abi(file):
    """
    获得abi对象
    :param file:
    :return:
    """
    with open(file, 'r') as f:
        all_str = f.read()
        abi_obj = json.loads(all_str)['abi']
    return abi_obj


def is_success_upload(receipt):
    """
    根据获得的 receipt 数据来判断上链是否成功
    :param receipt:
    :return:
    """
    if receipt.blockHash is None:
        return False
    else:
        return True


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 用于签名和验证签名
#  

"""
from web3 import Web3
import eth_account
import eth_utils
import eth_keys
from eth_keys.backends import NativeECCBackend
import os
import re
from eth_keys import KeyAPI
from eth_keys.backends import NativeECCBackend


def get_key_and_address(length=32):
    """
    生成16进制的private_key
    :param length:
    :return:
    """
    iv = os.urandom(length)
    private_key = eth_utils.to_hex(iv)
    public_Key = privatekey_to_publickey(private_key)
    address = privatekey_to_address(private_key)
    return private_key, address, public_Key


def privatekey_to_publickey(private_key):
    """
    生成私钥对应的公钥address
    :param private_key: 私钥 (hex或bytes）
    :return:
    """
    if not eth_utils.is_bytes(private_key):
        private_key = eth_utils.decode_hex(eth_utils.add_0x_prefix(private_key))
    pk = eth_keys.keys.PrivateKey(private_key)
    return pk.public_key


def privatekey_to_address(private_key):
    """
    根据私钥获得对应的公钥
    :param private_key: 私钥 （hex或bytes）
    :return:
    """
    if not eth_utils.is_bytes(private_key):
        private_key = eth_utils.decode_hex(eth_utils.add_0x_prefix(private_key))
    pk = eth_keys.keys.PrivateKey(private_key)
    return pk.public_key.to_checksum_address()


def sig_message(message, private_key_hex):
    """
    对message数据进行签名
    :param message: 需要签名的数据，(bytes)
    :param private_key_hex: 签名用的私钥
    :return:
    """
    private_key_byte = eth_utils.decode_hex(eth_utils.add_0x_prefix(private_key_hex))
    pk = eth_keys.keys.PrivateKey(private_key_byte)
    print('public key: {}'.format(pk.public_key))
    print('public key to address:  {}'.format(pk.public_key.to_checksum_address()))
    signature = pk.sign_msg(message)
    return signature


def ecdsa_sign(message: str, private_key_hex) -> str:
    """
    对message数据进行签名 (sig_message方法返回的结果相同)
    :param message: 需要签名的数据，(bytes)如果输入的不是byte会自动转成bytes
    :param private_key_hex: 签名用的私钥  str
    :return: str类型的签名数据 hex
    """
    if isinstance(message, str):
        message = message.encode('utf8')
    message_hash = eth_utils.keccak(message)
    keys_api = eth_keys.KeyAPI(NativeECCBackend())
    private_key_byte = eth_utils.decode_hex(eth_utils.add_0x_prefix(private_key_hex))
    pk = eth_keys.keys.PrivateKey(private_key_byte)
    signature = keys_api.ecdsa_sign(message_hash, pk)
    return str(signature).lstrip('0x')


def ecdsa_verify(message: str, signature: str, public_key):
    """
    验证签名是否正确
    :param message: 需要验证的数据 string
    :param signature: 签名 hex
    :param public_key: 必须是长的字符串，里面可能有前缀，需要处理  string or bytes
    :return:
    """
    if not isinstance(message, str):
        raise ValueError('message = {}  must string'.format(message))
    if not isinstance(signature, str):
        raise ValueError('signature = {}  must string'.format(signature))

    pb = check_public_key(public_key)   # pb 必须是一个没有 0x 和 04 前缀的 bytes
    message_hash = eth_utils.keccak(message.encode('utf8'))
    keys_api = eth_keys.KeyAPI(NativeECCBackend())
    signature_bytes = eth_utils.decode_hex(eth_utils.add_0x_prefix(signature))
    signature_obj = keys_api.Signature(signature_bytes=bytes.fromhex(signature))
    PublicKey = keys_api.ecdsa_recover(message_hash, signature_obj)
    # address_public = PublicKey.to_checksum_address()
    if '0x{}'.format(pb.decode()) == str(PublicKey):
        return True
    else:
        return False


def get_vrs_signature(signature_hex):
    """
    获得签名数据的 R S V
    :param signature_hex: 签名的数据 hex
    :return: (R:bytes, S:bytes, V:int)
    """
    from eth_keys import validation
    from eth_keys.exceptions import ValidationError, BadSignature
    signature_bytes = eth_utils.decode_hex(signature_hex)
    validation.validate_signature_bytes(signature_bytes)
    try:
        r = signature_bytes[0:32]
        s = signature_bytes[32:64]
        v = ord(signature_bytes[64:65])
    except ValidationError as err:
        raise BadSignature(str(err))
    return r, s, v
    # keys_api = KeyAPI(NativeECCBackend())
    # signature_obj = keys_api.Signature(signature_bytes=signature_bytes)
    # return signature_obj.vrs


def value_to_hex(bts):
    """
    将二进制转换为hex（16进制）
    :param bts:
    :return:
    """
    return eth_utils.to_hex(bts)


def check_public_key(public_key):
    """
    监测public_key 返回正确的数据
    请注意，某些库在字节序列化公钥前面加上前导\x04字节，在使用PublicKey对象之前必须将其删除 。
    有一些public key 是带有前缀的，以太坊的public key 需要去掉前缀
    :param public_key: 公钥 (str 或者 bytes)
    :return:  返回bytes的字符串
    """
    if isinstance(public_key, bytes):
        public_key = public_key.decode()

    if isinstance(public_key, str):
        public_key_no0x = re.sub('^0x','',public_key)
        if len(public_key_no0x) == 128:
            public_key_str = public_key_no0x
        elif len(public_key_no0x) == 130:
            public_key_str = public_key_no0x[2:]
        else:
            raise ValueError('string public_key={} length not 128'.format(public_key))

    return public_key_str.encode()


def kaccke(value):
    """
    获得value对应的keccak的hash值
    :param value:
    :return: bytes数据
    """
    from eth_utils import keccak
    return keccak(value)


def is_privatekey(key):
    """
    监测private key长度是否正确
    必须是64位的长度
    """
    import re
    prog = re.compile('^[a-zA-Z0-9]{64}$')
    tf = prog.match(key)
    return tf


def to_check_sum_address(address):
    """
    将 address 进行checkSum
    :param address:
    :return:
    """
    normal_address = eth_utils.add_0x_prefix(address)
    return Web3.toChecksumAddress(normal_address)

# def private_key_to_address(private_key):
#     """
#     获得 private_key 对应的 address (这个地址是checkSum的)
#     :param private_key:
#     :return: address (这个地址是checkSum的)
#     """
#     acct = eth_account.Account.privateKeyToAccount(private_key)
#     return acct.address


# def private_key_to_public_key(private_key):
#     """
#     获得private_key对应的public_key
#     :param private_key:
#     :return: public_key
#     """
#     keys = KeyAPI(NativeECCBackend)
#     privateKey = keys.PrivateKey(bytes.fromhex(eth_utils.remove_0x_prefix(private_key)))
#     return privateKey.public_key
#
#
# def public_key_to_address(public_key):
#     """
#     获得 public_key 对应的 address (这个地址是checkSum的)
#     :param public_key:
#     :return: address (这个地址是checkSum的)
#     """
#     keys = KeyAPI(NativeECCBackend)
#     publicKey = keys.PublicKey(bytes.fromhex(eth_utils.remove_0x_prefix(public_key)))
#     return publicKey.to_checksum_address()





#! /usr/bin/env python
# -*- coding: utf8 -*-

abi = """

"""

import json
with open('/Users/yuyongpeng/git/hard-chain/cport/ethr-did-registry/build/contracts/EthereumDIDRegistry.json') as f:
    jo = f.read()
    k = json.loads(jo)
    abi_list = k['abi']
    abi = json.dumps(abi_list)




# abi = """
# [{"constant": true, "inputs": [], "name": "currentMask", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getMinimumDeposit", "outputs": [{"name": "deposit", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getInitStarCoin", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "name", "outputs": [{"name": "", "type": "string"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_spender", "type": "address"}, {"name": "_value", "type": "uint256"}], "name": "approve", "outputs": [{"name": "", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "opsForAirDropPool", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getDailyLuckyNumber", "outputs": [{"name": "luckyNumber", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}, {"name": "_tokenAmount", "type": "uint256"}], "name": "transferFromOperatorPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_fansLockDuration", "type": "uint256"}], "name": "setFansLockDuration", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "totalSupply", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getRewardPool", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getMaximumDeposit", "outputs": [{"name": "deposit", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_from", "type": "address"}, {"name": "_to", "type": "address"}, {"name": "_value", "type": "uint256"}], "name": "transferFrom", "outputs": [{"name": "", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}, {"name": "_tokenAmount", "type": "uint256"}], "name": "transferFromStarPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "withdraw", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "getAirDropPool", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getFansActivateTime", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "decimals", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "operatorPool", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "depositToStarPool", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "activateTime", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getDailyRewardPool", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "opsForStarPool", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [], "name": "unpause", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getActivate", "outputs": [{"name": "status", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "airDropPool", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "initStarCoin", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "starLockedDuration", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getRewardScale", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [], "name": "claimOwnership", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_operator", "type": "address"}, {"name": "_state", "type": "uint8"}], "name": "setOpsForStarPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "maximumDeposit", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_maximumDeposit", "type": "uint256"}], "name": "setMaximumDeposit", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "opsForCentralBank", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "initEthForFans", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "paused", "outputs": [{"name": "", "type": "bool"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getStateStarPool", "outputs": [{"name": "state", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "dailyBlockNumber", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "minimumDeposit", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_spender", "type": "address"}, {"name": "_subtractedValue", "type": "uint256"}], "name": "decreaseApproval", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "rewardPool", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "dailyRewardPool", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "setInitStarCoin", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "nextDepositIndex", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "win_number", "type": "uint256"}], "name": "heartbeat", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "getOperatorPool", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_owner", "type": "address"}], "name": "balanceOf", "outputs": [{"name": "balance", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "initTimestamp", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getAccountOperator", "outputs": [{"name": "operator_address", "type": "address"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getCentralBank", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_operator", "type": "address"}, {"name": "_state", "type": "uint8"}], "name": "setOpsForRewardPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "getInitEthForFans", "outputs": [{"name": "ethForFans", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getFansLockDuration", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getStateRewardPool", "outputs": [{"name": "state", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [], "name": "destroy", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [], "name": "pause", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_rewardScale", "type": "uint256"}], "name": "setRewardScale", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "depositToAirDropPool", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "getLastUpdateMaskBlockNum", "outputs": [{"name": "blockNumber", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "opsForOperatorPool", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "owner", "outputs": [{"name": "", "type": "address"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "depositToCentralBank", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getStateAirDropPool", "outputs": [{"name": "state", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "symbol", "outputs": [{"name": "", "type": "string"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getCurrentMask", "outputs": [{"name": "mask", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getDepositAccount", "outputs": [{"name": "deposit", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "centralBank", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "depositIndex", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getStateOperatorPool", "outputs": [{"name": "state", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_dailyBlockNumber", "type": "uint256"}], "name": "setDailyBlockNumber", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "getDailyBlockNumber", "outputs": [{"name": "blockNumber", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "depositToOperatorPool", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_to", "type": "address"}, {"name": "_value", "type": "uint256"}], "name": "transfer", "outputs": [{"name": "", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "eth_address", "type": "address"}], "name": "getDepositIndex", "outputs": [{"name": "index", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "dailyLuckyNumber", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "deposit", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "activateBox", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "eth_address", "type": "address"}], "name": "getDepositAmount", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "opsForRewardPool", "outputs": [{"name": "", "type": "uint8"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "lastUpdateMaskBlockNum", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "starAvailable", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_dailyLuckyNumber", "type": "uint256"}], "name": "setDailyLuckyNumber", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "rewardScale", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_tokenAmount", "type": "uint256"}], "name": "allocateDailyRewardPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_operator", "type": "address"}, {"name": "_state", "type": "uint8"}], "name": "setOpsForCentralBank", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}], "name": "accountActivate", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "starPool", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_spender", "type": "address"}, {"name": "_addedValue", "type": "uint256"}], "name": "increaseApproval", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "fansLockDuration", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "totalDepositNumber", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_operator", "type": "address"}, {"name": "_state", "type": "uint8"}], "name": "setOpsForOperatorPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_initEthForFans", "type": "uint256"}], "name": "setInitEthForFans", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}], "name": "setAccountOperator", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "_account", "type": "address"}], "name": "getStateCentralBank", "outputs": [{"name": "state", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "_owner", "type": "address"}, {"name": "_spender", "type": "address"}], "name": "allowance", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "getStarPool", "outputs": [{"name": "amount", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [], "name": "pendingOwner", "outputs": [{"name": "", "type": "address"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "depositBox", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_minimumDeposit", "type": "uint256"}], "name": "setMinimumDeposit", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [{"name": "", "type": "address"}], "name": "lastWinBlockNumber", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"constant": false, "inputs": [{"name": "_operator", "type": "address"}, {"name": "_state", "type": "uint8"}], "name": "setOpsForAirDropPool", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "newOwner", "type": "address"}], "name": "transferOwnership", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_recipient", "type": "address"}], "name": "destroyAndSend", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}, {"name": "_status", "type": "uint8"}], "name": "setActivate", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_amount", "type": "uint256"}], "name": "depositToRewardPool", "outputs": [{"name": "success", "type": "bool"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": false, "inputs": [{"name": "_account", "type": "address"}, {"name": "_tokenAmount", "type": "uint256"}], "name": "transferFromCentralBank", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"constant": true, "inputs": [], "name": "lockedForStar", "outputs": [{"name": "", "type": "uint256"}], "payable": false, "stateMutability": "view", "type": "function"}, {"inputs": [{"name": "_name", "type": "string"}, {"name": "_decimals", "type": "uint8"}, {"name": "_symbol", "type": "string"}, {"name": "_operator", "type": "address"}, {"name": "_minimum_deposit", "type": "uint256"}, {"name": "_maximum_deposit", "type": "uint256"}, {"name": "_dailyLuckyNumber", "type": "uint256"}, {"name": "_operatorPool", "type": "uint256"}, {"name": "_rewardPool", "type": "uint256"}, {"name": "_airDropPool", "type": "uint256"}, {"name": "_starPool", "type": "uint256"}, {"name": "_contract_start_timestamp", "type": "uint256"}, {"name": "_starLockedDuration", "type": "uint256"}, {"name": "_rewardScale", "type": "uint256"}], "payable": false, "stateMutability": "nonpayable", "type": "constructor"}, {"payable": true, "stateMutability": "payable", "type": "fallback"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_winner", "type": "address"}, {"indexed": false, "name": "_value", "type": "uint256"}], "name": "WinStarCoin", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_depositor", "type": "address"}, {"indexed": false, "name": "_value", "type": "uint256"}], "name": "ToCentralBank", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_receiver", "type": "address"}, {"indexed": false, "name": "_value", "type": "uint256"}], "name": "FromCentralBank", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_sender", "type": "address"}, {"indexed": false, "name": "_block_number", "type": "uint256"}], "name": "HeartBeat", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_activator", "type": "address"}], "name": "Activate", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_depositor", "type": "address"}, {"indexed": false, "name": "_amount", "type": "uint256"}], "name": "Deposit", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "_withdrawor", "type": "address"}, {"indexed": false, "name": "_amount", "type": "uint256"}], "name": "WithDraw", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": false, "name": "_amount", "type": "uint256"}], "name": "ToDailyRewardPool", "type": "event"}, {"anonymous": false, "inputs": [], "name": "Pause", "type": "event"}, {"anonymous": false, "inputs": [], "name": "Unpause", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "previousOwner", "type": "address"}, {"indexed": true, "name": "newOwner", "type": "address"}], "name": "OwnershipTransferred", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "owner", "type": "address"}, {"indexed": true, "name": "spender", "type": "address"}, {"indexed": false, "name": "value", "type": "uint256"}], "name": "Approval", "type": "event"}, {"anonymous": false, "inputs": [{"indexed": true, "name": "from", "type": "address"}, {"indexed": true, "name": "to", "type": "address"}, {"indexed": false, "name": "value", "type": "uint256"}], "name": "Transfer", "type": "event"}]
# """



#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 14:44:00
LastEditors:  
LastEditTime: 2019-07-31 14:44:00
Description:  
"""





#! /usr/bin/env python
# -*- coding: utf8 -*-


import asyncio
from pycontractsdk.contracts import *
from pycontractsdk.mycontracts.cport import Cport
from pycontractsdk.contracts import Contract


async def eventlog_loop(event_filter, poll_interval, func):
    """

    :param event_filter:
    :param poll_interval:
    :param func:
    :return:
    """
    while True:
        for event in event_filter.get_new_entries():
            func(event)
        await asyncio.sleep(poll_interval)


def filter_event_async(provider, contract_address, abi, event_name, function, from_block=None, to_block='latest', timeout=2):
    """
    获得 event 的log
    :param provider: 具体连接的哪个server
    :param contract_address: 只能协约的address
    :param abi:
    :param event_name: 事件的名称，必须和abi中声明的一样
    :param function: 处理事件的函数
    :param from_block: 从哪个块开始
    :param to_block: 到哪个块结束
    :return: 获得的数据
    """
    cport = Cport(provider=provider, timeout=60, contract_address=contract_address, abi=abi)
    contract = cport.get_contract()
    event_filter = contract.events[event_name].createFilter(fromBlock=from_block, toBlock=to_block)
    loop = asyncio.get_event_loop()
    loop.run_until_complete(
        asyncio.gather(eventlog_loop(event_filter, timeout, function))
    )
    loop.close()


def filter_event(provider, contract_address, abi, event_name, function=None, from_block=None, to_block='latest'):
    """
    获得 event 的log
    :param provider: 具体连接的哪个server
    :param contract_address: 智能协约的address
    :param abi:
    :param event_name: 事件的名称，必须和abi中声明的一样
    :param function: 获得event之后执行的回调 (如果没有传递回调函数，就直接返回查询到的数据)
    :param from_block: 从哪个块开始
    :param to_block: 到哪个块结束
    :return: 获得的数据
    """
    cport = Cport(provider=provider, timeout=60, contract_address=contract_address, abi=abi)
    contract = cport.get_contract()
    # contract = cport.get_concise_contract()
    event_filter = contract.events[event_name].createFilter(fromBlock=from_block, toBlock=to_block)
    entries = event_filter.get_all_entries()
    print('fromBlock={}, toBlock={}'.format(from_block, to_block))
    if function is None:
        return entries
        # raise ValueError('必须设置handle function')
    function(entries)


def get_event_filter(provider, contract_address, abi, event_name, from_block=None, to_block='latest'):
    """
    获得 event_filter 对象
    :param provider: 具体连接的哪个server
    :param contract_address: 只能协约的address
    :param abi:
    :param event_name: 事件的名称，必须和abi中声明的一样
    :param from_block: 从哪个块开始
    :param to_block: 到哪个块结束
    :return: 获得的数据
    """
    cport = Cport(provider=provider, timeout=60, contract_address=contract_address, abi=abi)
    contract = cport.get_contract()
    event_filter = contract.events[event_name].createFilter(fromBlock=from_block, toBlock=to_block)
    return event_filter


class Event(Contract):
    def __init__(self, provider, timeout=60, *args, **kwargs):
        super(Event, self).__init__(provider, timeout, *args, **kwargs)

    def get_event(self, event_name, from_block, to_block):
        """
        获得 event 数据
        :param event_name:
        :param from_block:
        :param to_block:
        :return:
        """
        contract = self.get_contract()
        event_filter = contract.events[event_name].createFilter(fromBlock=from_block, toBlock=to_block)
        entries = event_filter.get_all_entries()
        return entries

















#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 14:44:55
LastEditors:  
LastEditTime: 2019-07-31 14:44:55
Description:  
"""
from pycontractsdk.contracts import Contract

# TODO: 这个方法不在使用，统一使用通用的父类
class Cport(Contract):
    """
    Cport 相关的协约操作
    """
    def __init__(self, provider, timeout=60, *args, **kwargs):
        super(Cport, self).__init__(provider, timeout, *args, **kwargs)

    def example(self, operator_address):
        """ 例子1 """
        contract = self.get_contract()
        value = contract.functions.getOperator(operator_address).call()
        return value

    def example2(self, operator_address):
        """ 例子2 """
        concise_contract = self.get_concise_contract()
        value = concise_contract.getOperator(operator_address)
        return value

    def change_owner(self, identity_address, new_owner_address):
        """
        调用Uport协约中的 changeOwner 方法
        :param identity_address: 参数1
        :param new_owner_address: 参数2
        :return:
        """
        contract = self.get_contract()
        uport_function = contract.functions.changeOwner(identity_address, new_owner_address)
        tf, tx_hash = self.upload_function(uport_function, self._private_key)
        # timeout = self.waitForTx(tx_hash)
        return tf, tx_hash

    def get_reward_pool(self):
        """
        获得协约里面的当日奖池的剩余
        """
        concise_contract = self.get_concise_contract()
        pool = int(concise_contract.getRewardPool() / 10000000000)
        return pool

    def set_operator(self, operator_address, value=1):
        """
        设置operator账号
        :param operator_address:
        :param value:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.setOperator(operator_address, value)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def get_operator(self, operator_address):
        """
        确认一个账号是否是operator账号
        :param operator_address:
        :return: 1:是，0:否
        """
        contract = self.get_contract()
        value = contract.functions.getOperator(operator_address).call()
        # concise_contract = self.get_concise_contract()
        # value = concise_contract.getOperator(operator_address)
        return value

    def change_contract_owner(self, new_owner_address):
        """
        变更owner账号（只有协约的拥有者能够调用）
        :param new_owner_address:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.changeContractOwner(new_owner_address)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def get_contract_owner(self):
        """
        获得协约的拥有者address
        :return: address  （hex）
        """
        concise_contract = self.get_concise_contract()
        value = concise_contract.getContractOwner()
        return value

    def set_delegate(self, attribute, delegate_address, value):
        """
        调用智能协约的：setDelegate 方法
        :param attribute:
        :param delegate_address:
        :param value:
        :return:
        """
        if not isinstance(value, int):
            raise ValueError('function:set_delegate(), value:[{0}] Incorrect parameter type'.format(value))
        contract = self.get_contract()
        function = contract.functions.setDelegate(attribute, delegate_address, value)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def get_delegate(self):
        """
        调用智能协约的 getDelegate 方法
        :return:
        """
        concise_contract = self.get_concise_contract()
        value = concise_contract.getDelegate()
        return value

    def identity_owner(self, identity_address):
        """
        调用智能协约的 identityOwner 的方法
        :param identity_address:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.identityOwner(identity_address)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def change_did_owner(self, identity_address, new_owner_address):
        """
        调用智能协约的 changeDIDOwner 方法
        :param identity_address:
        :param new_owner_address:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.changeDIDOwner(identity_address, new_owner_address)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def check_signature(self, identity_address, sigV, sigR, sigS, hash):
        """
        调用智能协约的 checkSignature 方法
        :param identity_address:
        :param sigV:
        :param sigR:
        :param sigS:
        :param hash:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.setAttributeSigned(identity_address, sigV, sigR, sigS, hash)
        tf, tx_hash = self.upload_function(function, self._private_key)
        return tf, tx_hash

    def set_attribute(self, identity_address, owner_address, name, value):
        """
        调用智能协约的 setAttribute() 方法
        :param identity_address:
        :param owner_address:
        :param name: bytes
        :param value: bytes
        :return:
        """
        contract = self.get_contract()
        function = getattr(contract.functions, 'setAttribute')(identity_address, owner_address, name, value)
        # function = contract.functions.setAttribute(identity_address, owner_address, name, value)
        tf, tx_hash = self.upload_function(function, self._private_key)
        # contract = self.get_concise_contract()
        # contract.functions.setAttribute(identity_address, owner_address, name, value).transact()
        return tf, tx_hash

    def set_attribute_signed(self, identity_address, sig_v, sig_r, sig_s, name, value):
        """
        调用智能协约的 setAttributeSigned() 方法
        :param identity_address:
        :param sig_v:
        :param sig_r:
        :param sig_s:
        :param name:
        :param value:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.setAttributeSigned(identity_address, sig_v, sig_r, sig_s, name, value)
        tf, tx_hash = self.upload_function(function, self._private_key)
        # tf, tx_hash = self.upload_function(function, self._owner_private_key)
        return tf, tx_hash


    def confirm_attribute(self, identity, name, value):
        """
        调用智能协约的 confirmAttribute 方法
        :param identity:
        :param name:
        :param value:
        :return:
        """
        contract = self.get_contract()
        function = contract.functions.confirmAttribute(identity, name, value)
        # TODO: onlyDelegate 才能调用，需要添加一个delegate的账号权限到属性中
        tf, tx_hash = self.upload_function(function, self._private_key)
        # tf, tx_hash = self.upload_function(function, self._operator_private_key)
        return tf, tx_hash


# -*- coding: utf-8 -*-

class Error(Exception):
    pass

class InputError(Error):
    def __index__(self, expression, message):
        self.expression = expression
        self.message = message

    def __str__(self):
        return repr()


#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 14:44:41
LastEditors:  
LastEditTime: 2019-07-31 14:44:41
Description:  
"""

from pycontractsdk.contracts import Contract


class Proof(Contract):
    def __init__(self, provider, timeout=60, *args, **kwargs):
        super(Proof, self).__init__(provider, timeout, *args, **kwargs)

    def submitProof(self, address, level, proofHash, metaData):
        """ submitProof """
        contract = self.get_contract()
        submitProof_function = contract.functions.submitProof(address, level, proofHash, metaData)
        tf, tx_hash = self.upload_function(submitProof_function, self._private_key)
        timeout = self.waitForTx(tx_hash)
        return tf, tx_hash



#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 14:45:25
LastEditors:  
LastEditTime: 2019-07-31 14:45:25
Description:  
"""
from pycontractsdk.contracts import Contract, keys
from web3.contract import ConciseContract


class StarCoin(Contract):
    """
    星钻 相关的协约操作
    """
    server_operator_private = ''        # 云端 python 上链的的账号
    contract_creater_private = ''       # 协约创建者的账号

    def __init__(self, provider, timeout=60, *args, **kwargs):
        super(StarCoin, self).__init__(provider, timeout, *args, **kwargs)
        self.server_operator_private = kwargs.pop('server_operator_private')
        self.contract_creater_private = kwargs.pop('contract_creater_private')

    def get_all_status(self):
        """ 获得 以太坊 和 协约 的当前状态 """
        print("_____________eth of contract {}_______________________________".format(self._contract_address))
        print("contract has {} eth".format(
            self.w3.eth.getBalance(self.w3.toChecksumAddress(self._contract_address))))
        # print out the parameters in imo one by one.
        # print("_____________print running state variables of contract {}______________".format(contract_address))
        # print("there are __ {} __ fans support star".format(contract.getDepositAmount()))
        print("contract nextDepositIndex is {}".format(self.call_concise_function('nextDepositIndex')))
        print("contract dailyLuckyNumber is {}".format(self.call_concise_function('dailyLuckyNumber')))

        print("contract current mask is {}".format(self.call_concise_function('getCurrentMask')))
        # print("account deposit amount is {}".format(contract.getDepositAmount(owner)))
        # print("account deposit index is {}".format(contract.getDepositIndex(owner)))

        print("------------print coin amount in seven pools--------------")
        # print("contract getOperatorPool is {}".format(self.call_concise_function('getOperatorPool') / 10 ** 10))
        # print("contract getRewardPool is {}".format(self.call_concise_function('getRewardPool') / 10 ** 10))
        # print("contract getAirDropPool is {}".format(self.call_concise_function('getAirDropPool') / 10 ** 10))
        # print("contract getCentralBank is {}".format(self.call_concise_function('getCentralBank') / 10 ** 10))
        # print("contract getDailyRewardPool is {}".format(self.call_concise_function('getDailyRewardPool') / 10 ** 10))

        print("contract getOperatorPool is \t {}".format(self.call_concise_function('getOperatorPool')))
        print("contract getRewardPool is \t {}".format(self.call_concise_function('getRewardPool')))
        print("contract getAirDropPool is \t {}".format(self.call_concise_function('getAirDropPool')))
        print("contract getCentralBank is \t {}".format(self.call_concise_function('getCentralBank')))
        print("contract getDailyRewardPool is \t {}".format(self.call_concise_function('getDailyRewardPool')))


        # check if one account is activated.
        print("------- 云端 python 程序账号的权限 ------------------------")
        server_operator_address = keys.privatekey_to_address(self.server_operator_private)
        print("account activate is \t {}".format(self.call_concise_function('getActivate', server_operator_address)))
        print("account state in account operator is \t {}".format(self.getAccountOperator(server_operator_address)))
        print("account state in operator pool is \t {}".format(
            self.call_concise_function('getStateOperatorPool', server_operator_address)))
        print("account state in reward pool is \t {}".format(
            self.call_concise_function('getStateRewardPool', server_operator_address)))
        print("account state in air drop pool is \t {}".format(
            self.call_concise_function('getStateAirDropPool', server_operator_address)))
        print("account state in central bank is \t {}".format(
            self.call_concise_function('getStateCentralBank', server_operator_address)))

        # check if one account is activated.
        print("------- 协约创建者 账号的权限 ------------------------")
        contract_creater_address = keys.privatekey_to_address(self.contract_creater_private)
        print("account activate is \t {}".format(self.call_concise_function('getActivate', contract_creater_address)))
        print("account state in account operator is \t {}".format(self.getAccountOperator(contract_creater_address)))
        print("account state in operator pool is \t {}".format(
            self.call_concise_function('getStateOperatorPool', contract_creater_address)))
        print("account state in reward pool is \t {}".format(
            self.call_concise_function('getStateRewardPool', contract_creater_address)))
        print("account state in air drop pool is \t {}".format(
            self.call_concise_function('getStateAirDropPool', contract_creater_address)))
        print("account state in central bank is \t {}".format(
            self.call_concise_function('getStateCentralBank', contract_creater_address)))

        print("------------------------------")
        print("reward scale is {}".format(self.call_concise_function('getRewardScale')))
        print("设备激活的时候给粉丝的eth数量: {}".format(self.call_concise_function('getInitEthForFans')))
        print("contract owner is {}".format(self.call_concise_function('owner')))
        print("当前预估每天产生的区块数量: {}".format(self.call_concise_function('getDailyBlockNumber')))
        print("总的押注的人数: {}".format(self.call_concise_function('getDepositAccount')))
        print("得到最近一次更新掩码的区块号: {}".format(self.call_concise_function('getLastUpdateMaskBlockNum')))
        print("最新的区块号: {}".format(self.last_block_number()))

    ####################################################################################################################
    ############################# 查询账号的权限 #########################################################################
    ####################################################################################################################
    def getStateOperatorPool(self, _account):
        """
        查询账号是否拥有 operator pool 的权限
        :param _account:    需要查询的账号
        :return:            0 未激活 1 激活 2 禁用
        """
        return self.call_concise_function('getStateOperatorPool', _account)

    def getStateRewardPool(self, _account):
        """
        查询账号是否拥有 reward pool 的权限
        :param _account:    需要查询的账号
        :return:            0 未激活 1 激活 2 禁用
        """
        return self.call_concise_function('getStateRewardPool', _account)

    def getStateAirDropPool(self, _account):
        """
        查询账号是否拥有 air drop pool 的权限
        :param _account:    需要查询的账号
        :return:            0 未激活 1 激活 2 禁用
        """
        return self.call_concise_function('getStateAirDropPool', _account)

    def getStateCentralBank(self, _account):
        """
        查询账号是否拥有 central bank 的权限
        :param _account:    需要查询的账号
        :return:            0 未激活 1 激活 2 禁用
        """
        return self.call_concise_function('getStateCentralBank', _account)

    def getAccountOperator(self, _account):
        """
        查询账号是否拥有 账号管理 的权限
        :param _account:    需要查询的账号
        :return:            0 未激活 1 激活 2 禁用
        """
        return self.call_concise_function('getAccountOperator', _account)

    ####################################################################################################################
    ############################# 得到每个币池的数 #######################################################################
    ####################################################################################################################
    def getOperatorPool(self):
        """
        查询 operator pool 的 钱数
        :return:   token 数量： 20 * 10 ** 10
        """
        return self.call_concise_function('getOperatorPool')

    def getRewardPool(self):
        """
        查询 reward pool 的 钱数
        :return:   token 数量： 20 * 10 ** 10
        """
        return self.call_concise_function('getRewardPool')

    def getAirDropPool(self):
        """
        查询 airDrop pool 的 钱数
        :return:   token 数量： 20 * 10 ** 10
        """
        return self.call_concise_function('getAirDropPool')

    def getCentralBank(self):
        """
        查询 centralBank pool 的 钱数
        :return:   token 数量： 20 * 10 ** 10
        """
        return self.call_concise_function('getCentralBank')

    def getDailyRewardPool(self):
        """
        查询 daily reward pool 的 钱数
        :return:   token 数量： 20 * 10 ** 10
        """
        return self.call_concise_function('getDailyRewardPool')

    ####################################################################################################################
    ############################# 获得一些设置的信息 ######################################################################
    ####################################################################################################################
    def getInitStarCoin(self):
        """
        激活的时候给粉丝的明星币数量
        :return:
        """
        return self.call_concise_function('getInitStarCoin')

    def getInitEthForFans(self):
        """
        得到激活的时候给粉丝的eth数量
        :return:
        """
        return self.call_concise_function('getInitEthForFans')

    def getDailyBlockNumber(self):
        """
        当前预估每天产生的区块数量
        :return:
        """
        return self.call_concise_function('getDailyBlockNumber')

    def getDepositAccount(self):
        """
        得到现在投注的总人数
        :return:
        """
        return self.call_concise_function('getDepositAccount')

    def getLastUpdateMaskBlockNum(self):
        """
        得到最近一次更新掩码的区块号
        :return:
        """
        return self.call_concise_function('getLastUpdateMaskBlockNum')

    def getDepositAmount(self, eth_address):
        """
        得到账号投注金额
        :param eth_address:     矿机的 address
        :return:
        """
        return self.call_concise_function('getDepositAmount', eth_address)

    def getDepositIndex(self, eth_address):
        """
        得到账号序号
        :param eth_address:     矿机的 address
        :return:
        """
        return self.call_concise_function('getDepositIndex', eth_address)

    def getCurrentMask(self):
        """
        得到现在掩码
        :return:
        """
        return self.call_concise_function('getCurrentMask')

    def getDailyLuckyNumber(self):
        """
        得到每天获得奖励的人数
        :return:
        """
        return self.call_concise_function('getDailyLuckyNumber')

    def getActivate(self, _account):
        """
        查询账户激活状态
        :param _account:    需要查询状态的账户
        :return:            0 未激活 1 激活 2 禁用 3 转移
        """
        return self.call_concise_function('getActivate', _account)

    def balanceOf(self, eth_address):
        """
        查询 矿机 的 token 数量 ，
        这个方法是在 https://github.com/OpenZeppelin/openzeppelin-contracts/blob/v1.12.0/contracts/token/ERC20/BasicToken.sol
        :param eth_address:    矿机的address
        :return:
        """
        return self.call_concise_function('balanceOf', eth_address)


    ####################################################################################################################
    ############################# 设置账号的权限 #########################################################################
    ####################################################################################################################
    def setOpsForAirDropPool(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 "空投管理" 账号, 有了这个权限就能 激活账号
        :param _account:    空投管理账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        print()
        print("---------------------------------------")
        print(" 设置 '空投' 管理账号 ")
        print("Before : 空投账号 {} status {}".format(_account, self.getStateAirDropPool(_account)))
        self.call_function_wait('setOpsForAirDropPool', _account, _status)
        print("After : 空投账号 {} status {}".format(_account, self.getStateAirDropPool(_account)))
        print("---------------------------------------")

    def setOpsForAirDropPool(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 "空投管理" 账号, 有了这个权限就能 激活账号
        :param _account:    空投管理账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        print()
        print("---------------------------------------")
        print(" 设置 '空投' 管理账号 ")
        print("Before : 空投账号 {} status {}".format(_account, self.getStateAirDropPool(_account)))
        self.call_function_wait('setOpsForAirDropPool', _account, _status)
        print("After : 空投账号 {} status {}".format(_account, self.getStateAirDropPool(_account)))
        print("---------------------------------------")

    def setOpsForRewardPool(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 "矿池管理账号" , 有了这个权限就能 从矿池打钱到每日挖矿池
        :param _account:    空投管理账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        print()
        print("---------------------------------------")
        print(" 设置 矿池 管理账号 ")
        print("Before : 空投账号 {} status {}".format(_account, self.getStateRewardPool(_account)))
        self.call_function_wait('setOpsForRewardPool', _account, _status)
        print("After : 空投账号 {} status {}".format(_account, self.getStateRewardPool(_account)))
        print("---------------------------------------")

    def setOpsForCentralBank(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 "中央银行管理账号" , 有了这个权限就能 从中央银行打钱到个人投注，从中央银行打钱到每日奖池，从中央银行转钱给个人
        :param _account:    运营账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        print()
        print("---------------------------------------")
        print(" 设置 '中央银行' 管理账号 ")
        print("Before : 中央银行管理账号 {} status {}".format(_account, self.getStateCentralBank(_account)))
        self.call_function_wait('setOpsForCentralBank', _account, _status)
        print("After : 中央银行管理账号 {} status {}".format(_account, self.getStateCentralBank(_account)))
        print("---------------------------------------")

    def setOpsForOperatorPool(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 "运营 operator" 账号, 有了这个权限就能 从运营账号转钱
        :param _account:    运营账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        print()
        print("---------------------------------------")
        print(" 设置 '运营' 管理账号 ")
        print("Before : 运营账号 {} status {}".format(_account, self.getStateOperatorPool(_account)))
        # self.w3.eth.functions.setOpsForOperatorPool( _account, _status)
        self.call_function_wait('setOpsForOperatorPool', _account, _status)
        print("After : 运营账号 {} status {}".format(_account, self.getStateOperatorPool(_account)))
        print("---------------------------------------")

    def setAccountOperator(self, _account, _status):
        """
        只有创建协约的账号才有权限调用
        设置 账号管理 账号, 有了这个权限就能 "设置奖励系数" "设置账户的激活状态"
        :param _account:    管理员账号 address
        :param _status:     0 未激活 1 激活 2 禁用
        :return:
        """
        # 权限判断 只有协约所有者才能调用

        print()
        print("---------------------------------------")
        print(" 设置 '账号管理' 账号 ")
        print("Before : {} status {}".format(_account, self.getAccountOperator(_account)))
        self.call_function_wait('setAccountOperator', _account, _status)
        print("After : {} status {}".format(_account, self.getAccountOperator(_account)))
        print("---------------------------------------")

    ####################################################################################################################
    ############################# 一些参数的设置 #########################################################################
    ####################################################################################################################
    def setRewardScale(self, _rewardScale):
        """
        设置奖励系数, 只有 设置了setAccountOperator的用才能调用
        :param _rewardScale:        系数值
        :param accountOperator:     默认使用 server 端 上链的operator账号
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyAccountOperator 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getAccountOperator(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 setRewardScal() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None

        print("Before : 奖励系数值为： {}".format(self.call_concise_function('getRewardScale')))
        self.call_function_wait('setRewardScale', _rewardScale)
        print("After : 奖励系数值为： {}".format(self.call_concise_function('getRewardScale')))
        print("---------------------------------------")

    def setActivate(self, _account, _status):
        """
        修改用户的激活状态， 只有 设置了setAccountOperator的用才能调用
        :param _account:            需要激活的矿机 address
        :param _status:             0 未激活 1 激活 2 禁用 3 转移
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyAccountOperator 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getAccountOperator(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 setRewardScal() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None

        print("Before : 用户的激活状态为： {}".format(self.getActivate(_account)))
        self.call_function_wait('setActivate', _account, _status)
        print("After : 用户的激活状态为： {}".format(self.getActivate(_account)))
        print("---------------------------------------")

    def setInitStarCoin(self, _amount):
        """
        修改激活时给矿机的明星币数量， 只有 设置了setAccountOperator的用才能调用
        :param _amount:             明星币数量 （默认值：1500 * 10 ** 10;）
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyAccountOperator 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getAccountOperator(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 setRewardScal() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        print("Before : 激活时给矿机的明星币数量为： {}".format(self.call_concise_function('getInitStarCoin')))
        self.call_function_wait('setInitStarCoin', _amount)
        print("After : 激活时给矿机的明星币数量为： {}".format(self.call_concise_function('getInitStarCoin')))
        print("---------------------------------------")

    def setDailyLuckyNumber(self, _dailyLuckyNumber):
        """
        修改每天得到奖励人数
        :param _dailyLuckyNumber:   每天获奖人数
        :return:
        """
        print()
        print("---------------------------------------")
        print("Before : 每天得到奖励人数为： {}".format(self.call_concise_function('getDailyLuckyNumber')))
        self.call_function_wait('setDailyLuckyNumber', _dailyLuckyNumber)
        print("After : 每天得到奖励人数为： {}".format(self.call_concise_function('getDailyLuckyNumber')))
        print("---------------------------------------")

    def setInitEthForFans(self, _initEthForFans):
        """
        设置激活的时候给粉丝的eth数量
        :param _initEthForFans:   给矿机的 eth 数量
        :return:
        """
        print()
        print("---------------------------------------")
        print("Before : 激活的时候给粉丝的eth数量为： {}".format(self.call_concise_function('getInitEthForFans')))
        self.call_function_wait('setInitEthForFans', _initEthForFans)
        print("After : 激活的时候给粉丝的eth数量为： {}".format(self.call_concise_function('getInitEthForFans')))
        print("---------------------------------------")

    def setDailyBlockNumber(self, _initEthForFans):
        """
        设置每天产生的区块数量
        :param _initEthForFans:   每天产生的区块数量
        :return:
        """
        print()
        print("---------------------------------------")
        print("Before : 每天产生的区块数量： {}".format(self.call_concise_function('getDailyBlockNumber')))
        self.call_function_wait('setDailyBlockNumber', _initEthForFans)
        print("After : 每天产生的区块数量： {}".format(self.call_concise_function('getDailyBlockNumber')))
        print("---------------------------------------")

    def accountActivate(self, _account):
        """
        激活一个账号   （onlyOpsForAirDropPool）
        :param _account:   矿机的address
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForAirDropPool 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateAirDropPool(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 accountActivate() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        print("Before : 账户的激活状态为: {}".format(self.getActivate(_account)))
        self.call_function_wait('accountActivate', _account)
        print("After : 账户的激活状态为： {}".format(self.getActivate(_account)))
        print("---------------------------------------")


    def centralBankToDeposit(self, _account, _tokenAmount):
        """
        从中央银行打钱到个人投注池  onlyOpsForCentralBank
        :param _account:        接收者的address
        :param _tokenAmount:    押注的钱数 10 * 10 ** 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForCentralBank 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateCentralBank(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 centralBankToDeposit() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None

        _account_checksum = keys.to_check_sum_address(_account)
        print("Before : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("Before : 个人投注池的钱数: {}".format(self.getDepositAmount(_account_checksum)))
        self.call_function_wait('centralBankToDeposit', _account_checksum, _tokenAmount)
        print("After : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("After : 个人投注池的钱数: {}".format(self.getDepositAmount(_account_checksum)))
        print("---------------------------------------")

    def centralBankToDailyRewardPoll(self, _tokenAmount):
        """
        从中央银行打钱到 当日奖池  onlyOpsForCentralBank
        :param _tokenAmount:    钱数 10 * 10 ** 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForCentralBank 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateCentralBank(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 centralBankToDailyRewardPoll() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        print("Before : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("Before : 当日奖池的钱数: {}".format(self.getDailyRewardPool()))
        self.call_function_wait('centralBankToDailyRewardPoll', _tokenAmount)
        print("After : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("After : 当日奖池的钱数: {}".format(self.getDailyRewardPool()))
        print("---------------------------------------")

    def allocateDailyRewardPool(self, _tokenAmount):
        """
        从挖矿总池子 ->  每天挖矿池
        :param _tokenAmount:   token 数量  1 * 10 ** 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForRewardPool 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateRewardPool(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 centralBankToDailyRewardPoll() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        print("Before : 挖矿总池的钱数为: {}".format(self.getRewardPool()))
        print("Before : 当日奖池的钱数: {}".format(self.getDailyRewardPool()))
        self.call_function_wait('allocateDailyRewardPool', _tokenAmount)
        print("After : 挖矿总池的钱数为: {}".format(self.getRewardPool()))
        print("After : 当日奖池的钱数: {}".format(self.getDailyRewardPool()))
        print("---------------------------------------")

    def transferFromRewardPool(self, _account, _tokenAmount):
        """
        从 rewardPool池 -> 个人账号   需要 onlyOpsForRewardPool 权限
        :param _account:        接收者的 address
        :param _tokenAmount:    钱数  1 * 10 * 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForRewardPool 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateRewardPool(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 transferFromOperatorPool() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        _account_checksum = keys.to_check_sum_address(_account)
        print("Before : rewardPool池的钱数为: {}".format(self.getRewardPool()))
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))
        self.call_function_wait('transferFromRewardPool', _account, _tokenAmount)
        print("After : rewardPool池的钱数为: {}".format(self.getRewardPool()))
        print("After : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))


    def transferFromOperatorPool(self, _account, _tokenAmount):
        """
        从 运营池 -> 个人账号   需要 onlyOpsForOperatorPool 权限
        :param _account:        接收者的 address
        :param _tokenAmount:    钱数  1 * 10 * 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForOperatorPool 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateOperatorPool(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 transferFromOperatorPool() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        _account_checksum = keys.to_check_sum_address(_account)
        print("Before : 运营池的钱数为: {}".format(self.getOperatorPool()))
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))
        self.call_function_wait('transferFromOperatorPool', _account, _tokenAmount)
        print("After : 运营池的钱数为: {}".format(self.getOperatorPool()))
        print("After : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))

    def transferFromCentralBank(self, _account, _tokenAmount):
        """
        中央银行 -> 个人账号   需要 onlyOpsForCentralBank 权限
        :param _account:        接收者的 address
        :param _tokenAmount:    钱数  1 * 10 * 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须有 onlyOpsForCentralBank 权限才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getStateCentralBank(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 transferFromCentralBank() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        _account_checksum = keys.to_check_sum_address(_account)
        print("Before : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))
        self.call_function_wait('transferFromCentralBank', _account, _tokenAmount)
        print("After : 中央银行的钱数为: {}".format(self.getCentralBank()))
        print("After : 个人账号的钱数: {}".format(self.balanceOf(_account_checksum)))


    def depositToOperatorPool(self, _amount):
        """
        个人账户 -> 运营池
        :param _amount:  token数量  1 * 10 ** 10
        :return:
        """
        print()
        print("---------------------------------------")
        # 获得发起交易者的账号
        private_key_address = keys.privatekey_to_address(self._private_key)
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("Before : 运营池的钱数为: {}".format(self.getOperatorPool()))
        self.call_function_wait('depositToOperatorPool', _amount)
        print("After : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("After : 运营池的钱数为: {}".format(self.getOperatorPool()))

    def depositToRewardPool(self, _amount):
        """
        个人账户  ->  总奖池
        :param _amount:
        :return:
        """
        print()
        print("---------------------------------------")
        # 获得发起交易者的账号
        private_key_address = keys.privatekey_to_address(self._private_key)
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("Before : 总奖池的钱数为: {}".format(self.getRewardPool()))
        self.call_function_wait('depositToRewardPool', _amount)
        print("After : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("After : 总奖池的钱数为: {}".format(self.getRewardPool()))


    def depositToAirDropPool(self, _amount):
        """
        个人账户  ->  空投池
        :param _amount:
        :return:
        """
        print()
        print("---------------------------------------")
        # 获得发起交易者的账号
        private_key_address = keys.privatekey_to_address(self._private_key)
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("Before : 空投池的钱数为: {}".format(self.getAirDropPool()))
        self.call_function_wait('depositToAirDropPool', _amount)
        print("After : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("After : 空投池的钱数为: {}".format(self.getAirDropPool()))


    def depositToCentralBank(self, _amount):
        """
        个人账户  ->  中央银行
        :param _amount:
        :return:
        """
        print()
        print("---------------------------------------")
        # 获得发起交易者的账号
        private_key_address = keys.privatekey_to_address(self._private_key)
        print("Before : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("Before : 中央银行的钱数为: {}".format(self.getCentralBank()))
        self.call_function_wait('depositToCentralBank', _amount)
        print("After : 个人账号的钱数: {}".format(self.balanceOf(private_key_address)))
        print("After : 中央银行的钱数为: {}".format(self.getCentralBank()))


    def heartbeat(self, win_number):
        """
        矿机挖矿  必须是激活账户才能调用
        :param win_number:
        :return:
        """
        print()
        print("---------------------------------------")
        # 发起交易的用户必须是 激活的账户 才能调用
        private_key_address = keys.privatekey_to_address(self._private_key)
        role = self.getActivate(private_key_address)
        if role != 1:
            print("发起交易者：{}  没有权限调用 heartbeat() 方法".format(self._private_key))
            self.get_account_role(private_key_address)
            return None
        self.call_function_wait('heartbeat', win_number)
        print("---------------------------------------")


    ####################################################################################################################
    ############################# 获得用户的权限信息 ######################################################################
    ####################################################################################################################

    def get_account_role(self, address):
        """
        获得账号的权限信息
        :param address:     需要查询的账号( public_key or private_key)
        :return:
        """
        # if keys.is_privatekey():
        #     checksum_address = keys.privatekey_to_address(address)
        # else:
        #     checksum_address = keys.to_check_sum_address(address)
        checksum_address = keys.to_check_sum_address(address)
        airDropPool_role = self.getStateAirDropPool(checksum_address)
        centralBankPool_role = self.getStateCentralBank(checksum_address)
        operatorPool_role = self.getStateOperatorPool(checksum_address)
        rewardPool_role = self.getStateRewardPool(checksum_address)
        accountOperator_role = self.getAccountOperator(checksum_address)
        print("账号：{}  拥有的权限：".format(checksum_address))
        print("airDropPool_role: {}".format(airDropPool_role))
        print("centralBankPool_role: {}".format(centralBankPool_role))
        print("operatorPool_role: {}".format(operatorPool_role))
        print("rewardPool_role: {}".format(rewardPool_role))
        print("accountOperator_role: {}".format(accountOperator_role))













































#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys
import os

here = os.path.abspath(os.path.dirname(__file__))
py_version = sys.version_info[:2]

if py_version <= (2, 7):
    raise RuntimeError('On Python 2, Oracle not Support !')
elif (3, 0) < py_version < (3, 6):
    raise RuntimeError('On Python 3, Oracle requires Python 3.6 or later')

try:
    README = open(os.path.join(here, 'README.md')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except:
    README = """
oracle 是一个智能协约的管理程序，用于导入event的数据到数据库中，和处理event数据
"""
    CHANGES = ''


setup(
    name="pycontractsdk.v2",
    version="0.1.0",
    author="yuyongpeng",
    author_email="yuyongpeng@hotmail.com",
    description=("This is a sdk of constract"),
    # long_description=README + '\n\n' + CHANGES,
    # long_description_markdown_filename='README.md',
    license="MIT",
    keywords="solidity web3 constract sdk",
    url="https://github.com/yuyongpeng/pyconstractsdk",
    # packages=['pycontractsdk'],  # 需要打包的目录列表
    packages = find_packages(where='.', exclude=('tests', 'tests.*'), include=('*',)),
    # 需要安装的依赖
    install_requires=[
        'web3==4.8.3',
        'ethereum==2.3.2',
        'eth-account==0.3.0',
        'eth-utils==1.4.1',
        'eth-keys==0.2.1',
        'eth-abi==1.3.0',
        'eth-keyfile==0.5.1',
        'eth-hash==0.2.0',
        'arrow==0.13.1',
        'logbook==1.4.3',
        'py-solc==3.2.0',
    ],
    py_modules=['pycontractsdk'],
    python_requires='>=3.6',
    setup_requires=['setuptools-markdown'],
    # 添加这个选项，在windows下Python目录的scripts下生成exe文件
    # 注意：模块与函数之间是冒号:
    entry_points={'console_scripts': [
    ]},

    # long_description=read('README.md'),
    classifiers=[  # 程序的所属分类列表
        "Development Status :: 3 - Alpha",
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Topic :: Utilities",
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
    ],
    zip_safe=False
)

